@@ -13,20 +13,21 @@
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-#ifndef BASIC_5_3_1_TAGGER_H
-#define BASIC_5_3_1_TAGGER_H
+#ifndef BASIC_5_3_2_TAGGER_H
+#define BASIC_5_3_2_TAGGER_H
 
-#include "analysis.h"
+#include "a.h"
+#include "lemma.h"
 
 #include <cstddef>
 #include <map>
 
 namespace Apertium {
-class basic_5_3_1_Tagger {
+class basic_5_3_2_Tagger {
 protected:
-  basic_5_3_1_Tagger();
-  std::map<Analysis, std::size_t> Model;
+  basic_5_3_2_Tagger();
+  std::map<a, std::map<Lemma, std::size_t> > Model;
 };
 }
 
-#endif // BASIC_5_3_1_TAGGER_H
+#endif // BASIC_5_3_2_TAGGER_H
